@@ -5,6 +5,7 @@ import tarfile
 import gzip
 import bz2
 import lzma
+import zipfile
 
 import jk_simpleexec
 import jk_logging
@@ -13,6 +14,7 @@ import jk_utils
 from .Spooler import Spooler
 from .SpoolInfo import SpoolInfo
 from .FileUncompressionGuess import FileUncompressionGuess
+from .impl import TARER
 
 
 
@@ -21,8 +23,6 @@ from .FileUncompressionGuess import FileUncompressionGuess
 
 class Unpacker(object):
 
-	_TAR_PATH = "/bin/tar"
-
 	################################################################################################################################
 	## Static Helper Methods
 	################################################################################################################################
@@ -119,21 +119,12 @@ class Unpacker(object):
 			srcTarFilePath = os.path.abspath(srcTarFilePath)
 			assert os.path.isfile(srcTarFilePath)
 			destDirPath = os.path.abspath(destDirPath)
-			if not os.path.isdir(destDirPath):
-				os.makedirs(destDirPath)
 
-			if not os.path.isfile(Unpacker._TAR_PATH):
-				raise Exception("'tar' not found!")
-
-			tarArgs = [
-				"-xf", srcTarFilePath, "-C", destDirPath
-			]
-			log2.notice("Invoking {} with: {}".format(Unpacker._TAR_PATH, str(tarArgs)))
-			cmdResult = jk_simpleexec.invokeCmd(Unpacker._TAR_PATH, tarArgs, workingDirectory=destDirPath)
-
-			if cmdResult.returnCode != 0:
-				cmdResult.dump(writeFunction=log2.error)
-				raise Exception("Failed to run 'tar'!")
+			TARER.untarToDir(
+				absTarFilePath = srcTarFilePath,
+				absDestDirPath = destDirPath,
+				log = log2,
+			)
 	#
 
 	#
@@ -277,6 +268,143 @@ class Unpacker(object):
 			return toFilePath
 	#
 
+	@staticmethod
+	def listTarContents(
+			*args,
+			tarFilePath:str,
+			log:jk_logging.AbstractLogger,
+		) -> typing.List[str]:
+
+		if args:
+			raise Exception("Invoke this method with named arguments only!")
+
+		assert isinstance(tarFilePath, str)
+
+		# ----
+
+		tarFilePath = os.path.abspath(tarFilePath)
+		assert os.path.isfile(tarFilePath)
+
+		return TARER.listTarContents(tarFilePath, log)
+	#
+
+	@staticmethod
+	def listZipContents(
+			*args,
+			zipFilePath:str,
+			log:jk_logging.AbstractLogger,
+			bIncludeDirectories:bool = True,
+		) -> typing.List[str]:
+
+		if args:
+			raise Exception("Invoke this method with named arguments only!")
+
+		assert isinstance(zipFilePath, str)
+
+		# ----
+
+		zipFilePath = os.path.abspath(zipFilePath)
+		assert os.path.isfile(zipFilePath)
+
+		ret = set()
+		with zipfile.PyZipFile(zipFilePath, "r") as zipf:
+			for relPath in zipf.namelist():
+				ret.add(relPath)
+				if bIncludeDirectories:
+					if "\\" in relPath:
+						raise Exception("????")
+					if "/" in relPath:
+						ret.add(os.path.dirname(relPath))
+
+		return sorted(ret)
+	#
+
+	@staticmethod
+	def listContents(
+			*args,
+			filePath:str,
+			packing:str,
+			log:jk_logging.AbstractLogger,
+		) -> typing.List[str]:
+
+		if packing == "tar":
+			return Unpacker.listTarContents(
+				*args,
+				tarFilePath=filePath,
+				log=log,
+			)
+		elif packing == "zip":
+			return Unpacker.listZipContents(
+				*args,
+				zipFilePath=filePath,
+				log=log,
+			)
+		else:
+			raise Exception("Invalid packaging identifier specified: {}".format(packing))
+	#
+
+	#
+	# Untar the specified <c>zip</c> archive and write the contents to the specified directory.
+	#
+	# @param	str srcZipFilePath					(required) The tar archive to read
+	# @param	str destDirFilePath					(required) The target directory to unpack all data to
+	# @param	AbstractLogger log					(required) A logger to write log information to
+	#
+	@staticmethod
+	def unzipToDir(
+			*args,
+			srcZipFilePath:str,
+			destDirPath:str,
+			log:jk_logging.AbstractLogger,
+		) -> None:
+
+		if args:
+			raise Exception("Invoke this method with named arguments only!")
+
+		assert isinstance(srcZipFilePath, str)
+		assert os.path.isfile(srcZipFilePath)
+		assert isinstance(destDirPath, str)
+		assert isinstance(log, jk_logging.AbstractLogger)
+
+		# ----
+
+		absDestDirPath = os.path.abspath(destDirPath)
+
+		with log.descend("Unpacking " + repr(srcZipFilePath) + " ...", logLevel=jk_logging.EnumLogLevel.NOTICE) as log2:
+			srcZipFilePath = os.path.abspath(srcZipFilePath)
+			assert os.path.isfile(srcZipFilePath)
+
+			with zipfile.PyZipFile(srcZipFilePath, "r") as zipf:
+				return zipf.extractall(absDestDirPath)
+	#
+
+	@staticmethod
+	def unpackToDir(
+			*args,
+			srcFilePath:str,
+			destDirPath:str,
+			packing:str,
+			log:jk_logging.AbstractLogger,
+		) -> None:
+
+		if packing == "tar":
+			Unpacker.untarToDir(
+				*args,
+				srcTarFilePath=srcFilePath,
+				destDirPath=destDirPath,
+				log=log,
+			)
+		elif packing == "zip":
+			Unpacker.unzipToDir(
+				*args,
+				srcZipFilePath=srcFilePath,
+				destDirPath=destDirPath,
+				log=log,
+			)
+		else:
+			raise Exception("Invalid packaging identifier specified: {}".format(packing))
+	#
+
 #
 
 
@@ -288,15 +416,5 @@ class Unpacker(object):
 
 
 
-
-
-
-
-
-
-
-
-
-
 
 
