@@ -1,8 +1,9 @@
-﻿
 
 
-__version__ = "0.2022.2.20"
+
 __api_version__ = 2
+__author__ = "Jürgen Knauth"
+__version__ = "0.2022.10.25"
 
 
 
@@ -14,4 +15,4 @@ from .Spooler import Spooler
 from .Packer import Packer
 from .Packer1 import Packer1
 from .Unpacker import Unpacker
-from .Unpacker1 import Unpacker1
\ No newline at end of file
+from .Unpacker1 import Unpacker1
